"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const api_js_1 = require("../api.js");
const queryParams_js_1 = require("../model/queryParams.js");
const helpers_js_1 = require("./helpers.js");
describe("Querying", async () => {
    let allPersons;
    let clientWithInactiveEntities;
    before(async () => {
        allPersons = (await helpers_js_1.client.persons.find()).data;
        const defaultConfig = (0, helpers_js_1.getDefaultConfig)();
        defaultConfig.queryOptions = {
            includeInactive: [queryParams_js_1.ActiveEntity.Affiliations, queryParams_js_1.ActiveEntity.Consents],
            includePersonsWithoutChurchAffiliation: true,
        };
        clientWithInactiveEntities = new api_js_1.CoreApiClient(defaultConfig);
    });
    beforeEach(async () => {
        await helpers_js_1.client.refreshTestData();
    });
    it("serializeFindQueryParams", async () => {
        const params = (0, queryParams_js_1.serializeFindQueryParams)({
            sort: [
                {
                    direction: "ascending",
                    path: "firstName",
                },
                "-lastName",
            ],
        });
        chai_1.assert.equal(params.get("sort"), "firstName,-lastName");
    });
    it("sorting", async () => {
        const result = await helpers_js_1.client.persons.find({
            sort: [
                {
                    direction: "ascending",
                    path: "displayName",
                },
            ],
            limit: 2,
            offset: 1,
        });
        chai_1.assert.equal(result.meta.total, allPersons.length);
        chai_1.assert.equal(result.meta.skipped, 1);
        chai_1.assert.equal(result.meta.limit, 2);
        chai_1.assert.instanceOf(result, api_js_1.WrappedWithMetaArrayPerson);
        const sortedPersons = allPersons.sort((a, b) => a.displayName > b.displayName ? 1 : -1);
        chai_1.assert.equal(result.data[0].uid, sortedPersons[1].uid);
        chai_1.assert.equal(result.data[1].uid, sortedPersons[2].uid);
    });
    it("filtering", async () => {
        const result = await helpers_js_1.client.persons.find({
            filter: {
                displayName: {
                    _gte: "Philly Daly",
                },
            },
        });
        const expectedResult = allPersons.filter((p) => p.displayName >= "Philly Daly");
        chai_1.assert.equal(result.meta.total, expectedResult.length);
    });
    it("filtering nested", async () => {
        const result = await helpers_js_1.client.persons.find({
            filter: {
                affiliations: {
                    person: {
                        displayName: {
                            _eq: "Philly Daly",
                        },
                    },
                },
            },
        });
        const expectedResult = allPersons.filter((p) => p.displayName >= "Philly Daly");
        chai_1.assert.equal(result.meta.total, expectedResult.length);
    });
    it("search - find persons using a fuzzy search", async () => {
        const result = await helpers_js_1.client.persons.find({
            search: "Philli dali",
        });
        chai_1.assert.isDefined(result);
        chai_1.assert.lengthOf(result.data, 1);
        chai_1.assert.equal(result.data[0].uid, helpers_js_1.person1Uid);
    });
    it("resolving", async () => {
        const result = await helpers_js_1.client.persons.find({
            fields: ["*", "affiliations.*"],
        });
        chai_1.assert.instanceOf(result.data[0].affiliations?.[0], api_js_1.Affiliation);
    });
    it("options - include persons without church affiliation", async () => {
        const result = await helpers_js_1.client.persons.find({
            options: {
                includePersonsWithoutChurchAffiliation: true,
            },
        });
        const resultUids = result.data.map((r) => r.uid);
        chai_1.assert.include(resultUids, helpers_js_1.personInactiveUid);
    });
    it("options - do not include persons without church affiliation", async () => {
        const result = await helpers_js_1.client.persons.find({
            options: {
                includePersonsWithoutChurchAffiliation: false,
            },
        });
        const resultUids = result.data.map((r) => r.uid);
        chai_1.assert.notInclude(resultUids, helpers_js_1.personInactiveUid);
    });
    it("options - include inactive entities", async () => {
        const result = await helpers_js_1.client.affiliations.find({
            options: {
                includeInactive: [queryParams_js_1.ActiveEntity.Affiliations],
            },
        });
        const inactiveAffiliation = result.data.find((a) => a.validFrom > new Date() || (a.validTo && a.validTo < new Date()));
        chai_1.assert.isDefined(inactiveAffiliation);
    });
    it("options - do not include inactive entities", async () => {
        const result = await helpers_js_1.client.affiliations.find();
        const inactiveAffiliation = result.data.find((a) => a.validFrom > new Date() || (a.validTo && a.validTo < new Date()));
        chai_1.assert.isUndefined(inactiveAffiliation);
    });
    it("options - include inactive by default with correct client", async () => {
        const result = await clientWithInactiveEntities.affiliations.find();
        const inactiveAffiliation = result.data.find((a) => a.validFrom > new Date() || (a.validTo && a.validTo < new Date()));
        chai_1.assert.isDefined(inactiveAffiliation);
    });
    it("options - include persons without church affiliation with correct client", async () => {
        const result = await clientWithInactiveEntities.persons.get(helpers_js_1.personInactiveUid);
        chai_1.assert.isDefined(result);
        chai_1.assert.equal(result.data.uid, helpers_js_1.personInactiveUid);
    });
});
